# encoding: utf-8

'''Pygubu-designer Virtual Events'''

FILE_NEW = '<<ACTION_FILE_NEW>>'
FILE_OPEN = '<<ACTION_FILE_OPEN>>'
FILE_SAVE = '<<ACTION_FILE_SAVE>>'
FILE_SAVEAS = '<<ACTION_FILE_SAVEAS>>'
FILE_QUIT = '<<ACTION_FILE_QUIT>>'
FILE_RECENT_CLEAR = '<<ACTION_FILE_RECENT_CLEAR>>'
TREE_NAV_UP = '<<ACTION_TREE_NAV_UP>>'
TREE_NAV_DOWN = '<<ACTION_TREE_NAV_DOWN>>'
TREE_ITEM_MOVE_DOWN = '<<ACTION_TREE_ITEM_MOVE_DOWN>>'
TREE_ITEM_MOVE_UP = '<<ACTION_TREE_ITEM_MOVE_UP>>'
TREE_ITEM_COPY = '<<ACTION_TREE_ITEM_COPY>>'
TREE_ITEM_PASTE = '<<ACTION_TREE_ITEM_PASTE>>'
TREE_ITEM_CUT = '<<ACTION_TREE_ITEM_CUT>>'
TREE_ITEM_DELETE = '<<ACTION_TREE_ITEM_DELETE>>'
TREE_ITEM_GRID_UP = '<<ACTION_TREE_ITEM_GRID_UP>>'
TREE_ITEM_GRID_DOWN = '<<ACTION_TREE_ITEM_GRID_DOWN>>'
TREE_ITEM_GRID_LEFT = '<<ACTION_TREE_ITEM_GRID_LEFT>>'
TREE_ITEM_GRID_RIGHT = '<<ACTION_TREE_ITEM_GRID_RIGHT>>'
TREE_ITEM_PREVIEW_TOPLEVEL = '<<ACTION_TREE_ITEM_PREVIEW_TOPLEVEL>>'
PREVIEW_TOPLEVEL_CLOSE_ALL = '<<ACTION_PREVIEW_TOPLEVEL_CLOSE_ALL>>'

